"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.changeBabelPreset = changeBabelPreset;
exports.updateBabelPreset = updateBabelPreset;
exports.createCompileBabelStep = createCompileBabelStep;

var _path = require("path");

var _ui = require("@sewing-kit/ui");

var _pluginUtilities = require("@sewing-kit/plugin-utilities");

const PRESET_MATCHER = /(babel-preset-shopify(?:\/[^.]*)?)/;

function normalizePreset(preset) {
  const match = preset.match(PRESET_MATCHER);
  return match ? match[1].replace('/index', '') : preset;
}

function createCheck(test) {
  return preset => {
    const normalized = normalizePreset(preset);
    return typeof test === 'string' ? test === normalized : test.some(test => test === normalized);
  };
}

function changeBabelPreset(from, to) {
  const check = createCheck(from);
  return ({
    presets = []
  }) => {
    for (const [index, preset] of presets.entries()) {
      if (typeof preset === 'string') {
        if (check(preset)) {
          presets[index] = to;
        }
      } else if (check(preset[0])) {
        preset[0] = to;
      }
    }
  };
}

function updateBabelPreset(match, options) {
  const check = createCheck(match);
  return ({
    presets = []
  }) => {
    for (const [index, preset] of presets.entries()) {
      if (typeof preset === 'string') {
        if (check(preset)) {
          presets[index] = [preset, options];
        }
      } else if (check(preset[0])) {
        preset[1] = { ...preset[1],
          ...options
        };
      }
    }
  };
}

function createCompileBabelStep(pkg, workspace, config, options) {
  return (0, _ui.createStep)(async step => {
    const {
      configFile = 'babel.js',
      outputPath
    } = options;
    const babelConfigPath = workspace.internal.configPath(`build/packages/${pkg.name}/${configFile}`);

    if (config.babelConfig == null) {
      throw new _pluginUtilities.MissingPluginError('@sewing-kit/plugin-babel');
    }

    await workspace.internal.write(babelConfigPath, `module.exports=${JSON.stringify((await config.babelConfig.promise({
      presets: []
    })))};`);
    const extensions = await config.extensions.promise([]);
    const sourceRoot = (0, _path.resolve)(pkg.root, 'src');
    await step.exec('node_modules/.bin/babel', [sourceRoot, ...(0, _pluginUtilities.toArgs)({
      outDir: outputPath,
      configFile: babelConfigPath,
      verbose: true,
      noBabelrc: true,
      babelConfig: false,
      extensions: extensions.join(',')
    }, {
      dasherize: true
    })]);
  });
}